//-----------------------------LICENSE NOTICE------------------------------------
//  This file is part of CPCtelera: An Amstrad CPC Game Engine
//  Copyright (C) 2015 ronaldo / Fremos / Cheesetea / ByteRealms (@FranGallegoBR)
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//------------------------------------------------------------------------------

#include <cpctelera.h>
#include "knight_spritesheet.h"
#include "enemy_demon_spritesheet.h"
#include "enemy_mage_spritesheet.h"
#include "map.h"
#include "map2.h"
#include "maptile.h"
#include "cajab.h"
#include "corazon.h"
#include "map3.h"
#include "llave.h"
#include "bomba.h"
#include "bala.h"
#include "explosion.h"
#include "rompible.h"

//COORDENADAS PARA LAS COLISIONES

#define UPLEFTX1 0
#define UPLEFTY1 0

#define UPLEFTX2 1
#define UPLEFTY2 -1

#define UPRIGHTX1 6
#define UPRIGHTY1 -1

#define UPRIGHTX2 7
#define UPRIGHTY2 0

#define DOWNRIGHTX1 7
#define DOWNRIGHTY1 15

#define DOWNRIGHTX2 6
#define DOWNRIGHTY2 17

#define DOWNLEFTX1 1
#define DOWNLEFTY1 17

#define DOWNLEFTX2 0
#define DOWNLEFTY2 15

//////////////////////////////////

//MOVIMIENTO

#define UP 1
#define DOWN 2
#define LEFT 3
#define RIGHT 4


#define VELOCITYH 1
#define VELOCITYV 2

//////////////////////////////////


#define VISIONX 25
#define VISIONY 50

#define SI 1
#define NO 0

#define ORIGEN_MAPA_Y   40
#define ORIGEN_MAPA cpctm_screenPtr(CPCT_VMEM_START, 0, ORIGEN_MAPA_Y)
#define ENEMIGOS  4

// ANIMACIONES

#define KNIGHT_FRAMES  12
#define ANIM_PAUSA      4
#define MOVE_FRAMES_R    4
#define MOVE_FRAMES_L    4
#define MOVE_FRAMES_U    2
#define MOVE_FRAMES_D    2

#define ENEMY_FRAMES  12
#define ENEMY_ANIM_PAUSA      2
#define ENEMY_MOVE_FRAMES_R    2
#define ENEMY_MOVE_FRAMES_L    2
#define ENEMY_MOVE_FRAMES_U    2
#define ENEMY_MOVE_FRAMES_D    2

//////////////////////////////////

//MAPAS 

#define N_MAPAS 3

//////////////////////////////////


//ARRAY DE MAPAS
u8* const array_mapas[N_MAPAS] = {g_map1, g_map2, g_map3};
extern u8 mapa_actual=0;
extern u8 explosion1 = 1;
extern u8 explosion2 = 1;
extern u8 explosion3 = 1;
extern u8 explosion4 = 1;

//ESTRUCTURA CAJA
typedef struct {
	u8 x, y;
	u8* sprite;
	u8 activo;

} TCaja;
//ESTRUCTURA ROMPIBLE
typedef struct {
	u8 x, y;
	u8* sprite;
	u8 activo;

} TRompible;
//ESTRUCTURA LLAVE
typedef struct {
	u8 x, y;
	u8* sprite;
	u8 activo;

} TLlave;
//ESTRUCTURA BOMBA
typedef struct {
	u8 x, y;
	u8* sprite;
	u8 activo;
	u8 temporizador;

} TBomba;
//ESTRUCTURA BALA
typedef struct {
	u8 x, y;
	u8 px, py;
	u8* sprite;
	u8 activo;
	u8 final;
	u8 vel;
	u8 dir;

} TBala;
//ESTRUCTURA CORAZON
typedef struct {
	u8 x, y;
	u8* sprite;
} TCorazon;
//ESTRUCTURA FRAMES
typedef struct {
   u8* sprite;
} TFrame;
//ESTRUCTURA PERSONAJE
typedef struct {
	u8 x, y;
	u8 px, py;
	u8* sprite;
	u8* move;
	u8 estado;
    TFrame* frame;
    u8 nframe;
    u8 lestado;
    u8 vida;
    u8 bombas;
    u8 muerto;
} Tpersonaje;
// Eventos del personaje
enum {
      EV_detenerse = 0
   ,  EV_moverse
} EEventos;

// Estados del personaje
enum {
      ST_quieto  = 0
   ,  ST_andando_r
   ,  ST_andando_l
   ,  ST_andando_u
   ,  ST_andando_d
   ,  ST_numEstados
} EEstados;

//ESTRUCTURA ENEMIGO
typedef struct {
	u8 x, y;
	u8 px, py;
	u8* sprite;
	u8* move;
	u8 estado;
    TFrame* frame;
    u8 nframe;
    u8 lestado;
    u8 siguiendo;
    u8 contAlerta;
    u8 randAlerta;
    u8 pared;
    u8 tipo;
    u8 muerto;
} Tenemigo;
// Eventos del enemigo
enum {
      EV_enemy_detenerse = 0
   ,  EV_enemy_moverse
} EnEventos;

// Estados del personaje
enum {
      ST_enemy_quieto  = 0
   ,  ST_enemy_andando_r
   ,  ST_enemy_andando_l
   ,  ST_enemy_andando_u
   ,  ST_enemy_andando_d
   ,  ST_enemy_numEstados
} EnEstados;

const TFrame g_frames[KNIGHT_FRAMES] = {
      { g_tile_knight_spritesheet_00 }   ,  { g_tile_knight_spritesheet_01 }
   ,  { g_tile_knight_spritesheet_02 }   ,  { g_tile_knight_spritesheet_03 }
   ,  { g_tile_knight_spritesheet_04 }   ,  { g_tile_knight_spritesheet_05 }
   ,  { g_tile_knight_spritesheet_06 }   ,  { g_tile_knight_spritesheet_07 }
   ,  { g_tile_knight_spritesheet_08 }   ,  { g_tile_knight_spritesheet_09 }
   ,  { g_tile_knight_spritesheet_12 }	 ,  { g_tile_knight_spritesheet_13 }
};
const TFrame g_enemy_frames[ENEMY_FRAMES] = {
      { g_tile_enemy_demon_spritesheet_00 }   ,  { g_tile_enemy_demon_spritesheet_01 }
   ,  { g_tile_enemy_demon_spritesheet_02 }   ,  { g_tile_enemy_demon_spritesheet_03 }
   ,  { g_tile_enemy_demon_spritesheet_04 }   ,  { g_tile_enemy_demon_spritesheet_05 }
   ,  { g_tile_enemy_demon_spritesheet_06 }   ,  { g_tile_enemy_demon_spritesheet_07 }
   ,  { g_tile_enemy_demon_spritesheet_08 }   ,  { g_tile_enemy_demon_spritesheet_09 }
   ,  { g_tile_enemy_demon_spritesheet_12 }	  ,  { g_tile_enemy_demon_spritesheet_13 }
};


const TFrame g_enemy2_frames[ENEMY_FRAMES] = {
      { g_tile_enemy_mage_spritesheet_00 }   ,  { g_tile_enemy_mage_spritesheet_01 }
   ,  { g_tile_enemy_mage_spritesheet_02 }   ,  { g_tile_enemy_mage_spritesheet_03 }
   ,  { g_tile_enemy_mage_spritesheet_04 }   ,  { g_tile_enemy_mage_spritesheet_05 }
   ,  { g_tile_enemy_mage_spritesheet_06 }   ,  { g_tile_enemy_mage_spritesheet_07 }
   ,  { g_tile_enemy_mage_spritesheet_08 }   ,  { g_tile_enemy_mage_spritesheet_09 }
   ,  { g_tile_enemy_mage_spritesheet_12 }	  , { g_tile_enemy_mage_spritesheet_13 }
};


TFrame* const anim_andar_r[MOVE_FRAMES_R] = { &g_frames[0], &g_frames[1], &g_frames[2], &g_frames[3]};
TFrame* const anim_andar_l[MOVE_FRAMES_L] = { &g_frames[4], &g_frames[5], &g_frames[6], &g_frames[7]};
TFrame* const anim_andar_u[MOVE_FRAMES_U] = { &g_frames[10], &g_frames[11]};
TFrame* const anim_andar_d[MOVE_FRAMES_D] = { &g_frames[8], &g_frames[9]};

TFrame* const enemy_anim_andar_r[ENEMY_MOVE_FRAMES_R] = { &g_enemy_frames[0], &g_enemy_frames[1]};
TFrame* const enemy_anim_andar_r2[ENEMY_MOVE_FRAMES_R] = { &g_enemy2_frames[0], &g_enemy2_frames[1]};
TFrame* const enemy_anim_andar_l[ENEMY_MOVE_FRAMES_L] = { &g_enemy_frames[4], &g_enemy_frames[5]};
TFrame* const enemy_anim_andar_l2[ENEMY_MOVE_FRAMES_L] = { &g_enemy2_frames[4], &g_enemy2_frames[5]};
TFrame* const enemy_anim_andar_u[ENEMY_MOVE_FRAMES_U] = { &g_enemy_frames[10], &g_enemy_frames[11]};
TFrame* const enemy_anim_andar_u2[ENEMY_MOVE_FRAMES_U] = { &g_enemy2_frames[10], &g_enemy2_frames[11]};
TFrame* const enemy_anim_andar_d[ENEMY_MOVE_FRAMES_D] = { &g_enemy_frames[8], &g_enemy_frames[9]};
TFrame* const enemy_anim_andar_d2[ENEMY_MOVE_FRAMES_D] = { &g_enemy2_frames[8], &g_enemy2_frames[9]};

Tpersonaje personaje;
Tenemigo enemigo1;
Tenemigo enemigo2;
Tenemigo enemigo3;
Tenemigo enemigo4;
TLlave llave;
TBala bala;
TBomba bomba;
TBomba explosion;
TCaja caja;
TRompible pared1;
TRompible pared2;
TRompible pared3;
TCorazon corazon;
u8* mapa;
Tenemigo* enemigos[4];
TRompible* paredes[3];
cpctm_createTransparentMaskTable (g_tablatrans, 0x0200, M0, 0);

void andando_r(){if(++personaje.nframe == MOVE_FRAMES_R*ANIM_PAUSA){personaje.nframe = 0;}}
void andando_l(){if(++personaje.nframe == MOVE_FRAMES_L*ANIM_PAUSA){personaje.nframe = 0;}}
void andando_u(){if(++personaje.nframe == MOVE_FRAMES_U*ANIM_PAUSA){personaje.nframe = 0;}}
void andando_d(){if(++personaje.nframe == MOVE_FRAMES_D*ANIM_PAUSA){personaje.nframe = 0;}}
void asignarFrame(TFrame **animacion) {personaje.frame = animacion[personaje.nframe / ANIM_PAUSA];}
void en_andando_r(Tenemigo* enemigo){if(++enemigo->nframe == ENEMY_MOVE_FRAMES_R*ENEMY_ANIM_PAUSA){enemigo->nframe = 0;}}
void en_andando_l(Tenemigo* enemigo){if(++enemigo->nframe == ENEMY_MOVE_FRAMES_L*ENEMY_ANIM_PAUSA){enemigo->nframe = 0;}}
void en_andando_u(Tenemigo* enemigo){if(++enemigo->nframe == ENEMY_MOVE_FRAMES_U*ENEMY_ANIM_PAUSA){enemigo->nframe = 0;}}
void en_andando_d(Tenemigo* enemigo){if(++enemigo->nframe == ENEMY_MOVE_FRAMES_D*ENEMY_ANIM_PAUSA){enemigo->nframe = 0;}}
void en_asignarFrame(TFrame **animacion, Tenemigo* enemigo) { enemigo->frame = animacion[enemigo->nframe / ANIM_PAUSA];}
void selectSprite() {
   switch(personaje.estado) {
      case ST_quieto:      { personaje.frame = &g_frames[8];  break; }
      case ST_andando_r:     { asignarFrame(anim_andar_r);    break; }
      case ST_andando_l:   { asignarFrame(anim_andar_l);   break; }
      case ST_andando_u:    { asignarFrame(anim_andar_u); break; }
      case ST_andando_d:     { asignarFrame(anim_andar_d); break; }
   }
}

void selectEnSprite(Tenemigo* enemigo) {
   switch(enemigo->estado) {
      case ST_enemy_quieto:      { if(enemigo->tipo == 1){enemigo->frame = &g_enemy_frames[8];}else {enemigo->frame = &g_enemy2_frames[8];}  break; }
      case ST_enemy_andando_r:     { if(enemigo->tipo == 1){en_asignarFrame(enemy_anim_andar_r,enemigo);} else{en_asignarFrame(enemy_anim_andar_r2,enemigo);}  break; }
      case ST_enemy_andando_l:   { if(enemigo->tipo == 1){  en_asignarFrame(enemy_anim_andar_l,enemigo);  } else{en_asignarFrame(enemy_anim_andar_l2,enemigo);}  break; }
      case ST_enemy_andando_u:    { if(enemigo->tipo == 1){ en_asignarFrame(enemy_anim_andar_u,enemigo); } else{en_asignarFrame(enemy_anim_andar_u2,enemigo);}  break; }
      case ST_enemy_andando_d:     { if(enemigo->tipo == 1){en_asignarFrame(enemy_anim_andar_d,enemigo);} else{en_asignarFrame(enemy_anim_andar_d2,enemigo);}  break; }
   }
}

void stateExe() {
   switch(personaje.estado) {
      case ST_andando_r:   andando_r();     break;
      case ST_andando_l:   andando_l();   break;
      case ST_andando_u:   andando_u();    break;
      case ST_andando_d:   andando_d();     break;
   }
}

void stateEnExe(Tenemigo* enemigo) {
   switch(enemigo->estado) {
      case ST_enemy_andando_r:   en_andando_r(enemigo);   break;
      case ST_enemy_andando_l:   en_andando_l(enemigo);   break;
      case ST_enemy_andando_u:   en_andando_u(enemigo);   break;
      case ST_enemy_andando_d:   en_andando_d(enemigo);   break;
   }
}


u8* obtenerTilePtr(u8 x, u8 y) {  return mapa + y/4*g_map1_W + x/2;}
u8 comprobarColision(u8* tileSuelo1, u8* tileSuelo2){
	if(    (*tileSuelo1 <  54 || *tileSuelo2 <  54) 
		&& (*tileSuelo1 != 10 || *tileSuelo2 != 10) 
		&& (*tileSuelo1 != 13 || *tileSuelo2 != 13) 
		&& (*tileSuelo1 != 16 || *tileSuelo2 != 16) 
		&& (*tileSuelo1 != 37 || *tileSuelo2 != 37) 
		&& (*tileSuelo1 != 40 || *tileSuelo2 != 40) 
		&& (*tileSuelo1 != 43 || *tileSuelo2 != 43)){
		return SI;
	}else return NO;
}
void drawBala(){
	u8* pvmem = cpct_getScreenPtr(CPCT_VMEM_START, bala.x, bala.y+ORIGEN_MAPA_Y);
	cpct_drawSprite (bala.sprite, pvmem, G_BALA_W, G_BALA_H);
}

void deleteBala(){
	u8 w = 4 + (bala.px & 1);
	u8 h = 4 + (bala.py & 3 ? 1: 0);
	cpct_etm_drawTileBox2x4 (bala.px / 2, bala.py / 4, w,h, g_map1_W, ORIGEN_MAPA, mapa);
}

void reDrawBala(){
	deleteBala();
	bala.px = bala.x;
	bala.py = bala.y;
	drawBala();
}
void dibujarRompible(){
	u8* pvmem;
	u8 i;
	for(i=0; i<3; i++){
		if(paredes[i]->activo == SI){
			pvmem = cpct_getScreenPtr(CPCT_VMEM_START, paredes[i]->x, paredes[i]->y+ORIGEN_MAPA_Y);
			cpct_drawSprite (paredes[i]->sprite, pvmem, G_ROMPIBLE_W, G_ROMPIBLE_H);
		}
	}
}
void borrarRompible(TRompible* pared){
	cpct_etm_drawTileBox2x4 (pared->x / 2, pared->y / 4, 4,5, g_map1_W, ORIGEN_MAPA, mapa);
}

void disparar(){
	bala.sprite =  g_bala;
	bala.activo = SI;
	if(personaje.estado == ST_andando_u){
		bala.x = personaje.x+4;
		bala.y = personaje.y-6;
		bala.vel = -6;
		if(bala.y-60  < 0){ bala.final = 6; } else bala.final = bala.y-60;
		bala.dir = UP;
	}else if(personaje.estado == ST_andando_d){
		bala.x = personaje.x+4;
		bala.y = personaje.y+14;
		bala.vel = 6;
		bala.final = bala.y+60;
		bala.dir = DOWN;
	}else if(personaje.estado == ST_andando_l){
		bala.x = personaje.x-6;
		bala.y = personaje.y+6;
		bala.vel = -3;
		if(bala.x-30 < 0){ bala.final = 3; }else bala.final = bala.x-30;
		bala.dir = LEFT;
	}else{
		bala.x = personaje.x+8;
		bala.y = personaje.y+6;
		bala.vel = 3;
		bala.final = bala.x+30;
		bala.dir = RIGHT;
	}
}
/////////////////////////////////////DIBUJAR PERSONAJE\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\

void drawEn(Tenemigo* enemigo){
	u8* memptr = cpct_getScreenPtr(CPCT_VMEM_START, enemigo->x, enemigo->y+ORIGEN_MAPA_Y);
	cpct_drawSpriteMaskedAlignedTable (enemigo->frame->sprite, memptr, 8, 16, g_tablatrans);
}

void deleteEn(Tenemigo* enemigo){
	u8 w = 4 + (enemigo->px & 1);
	u8 h = 4 + (enemigo->py & 3 ? 1: 0);
	cpct_etm_drawTileBox2x4 (enemigo->px / 2, enemigo->py / 4, w,h, g_map1_W, ORIGEN_MAPA, mapa);
}
void reDrawEn(Tenemigo* enemigo){
	deleteEn(enemigo);
	enemigo->px = enemigo->x;  
	enemigo->py = enemigo->y;
	drawEn(enemigo);
}
void compruebaBala(){
	u8 i;
	u8* tileSuelo = obtenerTilePtr(bala.x, bala.y);

	for ( i = 0; i < 2; ++i){
		if (bala.x >= enemigos[i]->x && bala.x <= enemigos[i]->x+16 && bala.y >= enemigos[i]->y && bala.y <= enemigos[i]->y+16){
			enemigos[i]->muerto = SI;
			bala.activo = NO;
			deleteEn(enemigos[i]);
			deleteBala();
			enemigos[i]->x = -10;
		}else if(bala.x >= paredes[i]->x && bala.x <= paredes[i]->x+16 && bala.y >= paredes[i]->y && bala.y <= paredes[i]->y+16){
			bala.activo = NO;
			deleteBala();
		}
	}

	if(bala.x <=1 || bala.x >= 76 || bala.y <=2 || bala.y >= 150){bala.activo = NO; deleteBala();}

	if(bala.activo == SI && comprobarColision(tileSuelo, tileSuelo) == SI){
		bala.activo = NO;
		deleteBala();
	}else{
		if(bala.dir == UP){
			bala.y+=bala.vel;

			if(bala.y <= bala.final){
				bala.activo = NO;
				deleteBala();
			}
		}else if(bala.dir == DOWN){
			bala.y+=bala.vel;

			if(bala.y >= bala.final){
				bala.activo = NO;
				deleteBala();
			}
		}else if(bala.dir == LEFT){
			bala.x+=bala.vel;

			if(bala.x <= bala.final){
				bala.activo = NO;
				deleteBala();
			}
		}else if(bala.dir == RIGHT){
			bala.x+=bala.vel;

			if(bala.x >= bala.final){
				bala.activo = NO;
				deleteBala();
			}
		}
	}
}

void ponerBomba(){
	bomba.x = personaje.x+1;
	bomba.y = personaje.y;
	bomba.activo = SI;
	bomba.temporizador = 0;
	personaje.bombas--;
}

void compruebaExplosion(){
	u8* tileSuelo1 = obtenerTilePtr(bomba.x, bomba.y-5);
	u8* tileSuelo2 = obtenerTilePtr(bomba.x+10, bomba.y);
	u8* tileSuelo3 = obtenerTilePtr(bomba.x, bomba.y+20);
	u8* tileSuelo4 = obtenerTilePtr(bomba.x-5, bomba.y);

		if(bomba.y-5 <= 0 || comprobarColision(tileSuelo1, tileSuelo1) == SI) explosion1 = 0; 
		else {explosion1 = 1;}
		if(comprobarColision(tileSuelo2, tileSuelo2) == SI) explosion2 = 0; 				  
		else {explosion2 = 1;}
		if(comprobarColision(tileSuelo3, tileSuelo3) == SI) explosion3 = 0; 				  
		else {explosion3 = 1;}
		if(bomba.x-5 <= 0 || comprobarColision(tileSuelo4, tileSuelo4) == SI) explosion4 = 0; 
		else {explosion4 = 1;}
}

void explotaBomba(){
	u8 i;
	explosion.activo = SI;
	explosion.temporizador = 0;
	explosion.x = bomba.x;
	explosion.y = bomba.y;

	compruebaExplosion();

	for ( i = 0; i < 3; ++i){
		if(paredes[i]->activo == SI && (bomba.x >= paredes[i]->x && bomba.x <= paredes[i]->x+5) && (bomba.y-16 >= paredes[i]->y && bomba.y-16 <= paredes[i]->y+16)){
			paredes[i]->activo = NO;
			borrarRompible(paredes[i]);
		}
		if(paredes[i]->activo == SI && (bomba.x+12 >= paredes[i]->x && bomba.x+12 <= paredes[i]->x+5) && (bomba.y+4 >= paredes[i]->y && bomba.y <= paredes[i]->y+16)){
			paredes[i]->activo = NO;
			borrarRompible(paredes[i]);
		}
		if(paredes[i]->activo == SI && (bomba.x >= paredes[i]->x && bomba.x <= paredes[i]->x+5) && (bomba.y+32 >= paredes[i]->y && bomba.y+32 <= paredes[i]->y+16)){
			paredes[i]->activo = NO;
			borrarRompible(paredes[i]);
		}
		if(paredes[i]->activo == SI && (bomba.x-6 >= paredes[i]->x && bomba.x-6 <= paredes[i]->x+5) && (bomba.y+2 >= paredes[i]->y && bomba.y <= paredes[i]->y+16)){
			paredes[i]->activo = NO;
			borrarRompible(paredes[i]);
		}
	}

	if(personaje.x+6 >= explosion.x-6 && personaje.x <= explosion.x+12 && personaje.y >= explosion.y && personaje.y <= explosion.y 
		|| personaje.x >= explosion.x && personaje.x <= explosion.x && personaje.y+16 >= explosion.y-16 && personaje.y <= explosion.y+32 ){
		personaje.muerto = SI;
		deletePj();
	}	

	for ( i = 0; i < 2; ++i){
		if(personaje.x+6 >= explosion.x-6 && personaje.x <= explosion.x+12 && personaje.y >= explosion.y && personaje.y <= explosion.y 
			|| personaje.x >= explosion.x && personaje.x <= explosion.x && personaje.y+16 >= explosion.y-16 && personaje.y <= explosion.y+32){
			enemigos[i]->muerto = SI;
			deleteEn();
		}	
	}

	cpct_etm_drawTileBox2x4 (explosion.x / 2, explosion.y / 4, 4,5, g_map1_W, ORIGEN_MAPA, mapa);

}

void compruebaBomba(){
	if(bomba.temporizador >= 50){
		bomba.activo = NO;
		explotaBomba();
		borrarBomba();
	}else bomba.temporizador++;
}

void drawMapa(){cpct_etm_drawTilemap2x4(g_map1_W, g_map1_H, ORIGEN_MAPA, mapa);}
void dibujarCaja(){
	u8* pvmem = cpct_getScreenPtr(CPCT_VMEM_START, caja.x, caja.y+ORIGEN_MAPA_Y);
	cpct_drawSprite (caja.sprite, pvmem, G_CAJAB_W, G_CAJAB_H);
}
void dibujarCorazones(){
	u8* pvmem;
	u8 i;
	for (i = 0; i < personaje.vida; i++){
		pvmem = cpct_getScreenPtr(CPCT_VMEM_START, corazon.x+(i*7), corazon.y);
		cpct_drawSprite (corazon.sprite, pvmem, G_CORAZON_W, G_CORAZON_H);
	}
}
void borrarCaja(){
	u8* memptr;
	cpct_etm_drawTileBox2x4 (caja.x / 2, caja.y / 4, 4,5, g_map1_W, ORIGEN_MAPA, mapa);
}


void dibujarLlave(){
	u8* pvmem = cpct_getScreenPtr(CPCT_VMEM_START, llave.x, llave.y+ORIGEN_MAPA_Y);
	cpct_drawSpriteMaskedAlignedTable (llave.sprite, pvmem, 6, 16, g_tablatrans);
}
void borrarLlave(){
	cpct_etm_drawTileBox2x4 (llave.x / 2, llave.y / 4, 4,5, g_map1_W, ORIGEN_MAPA, mapa);
}
void dibujarBomba(){
	u8* pvmem = cpct_getScreenPtr(CPCT_VMEM_START, bomba.x, bomba.y+ORIGEN_MAPA_Y);
	cpct_drawSpriteMaskedAlignedTable (bomba.sprite, pvmem, 6, 16, g_tablatrans);
}
void borrarBomba(){
	cpct_etm_drawTileBox2x4 (bomba.x / 2, bomba.y / 4, 4,5, g_map1_W, ORIGEN_MAPA, mapa);
}
void dibujarExplosion(){

	if(explosion.temporizador >= 10){
		explosion.activo = NO;
		borrarExplosion();
	}else{
		u8* pvmem = cpct_getScreenPtr(CPCT_VMEM_START, explosion.x, explosion.y+ORIGEN_MAPA_Y);
		cpct_drawSpriteMaskedAlignedTable (explosion.sprite, pvmem, 6, 16, g_tablatrans);
		
		if(explosion1 == 1){
			pvmem = cpct_getScreenPtr(CPCT_VMEM_START, explosion.x, explosion.y+ORIGEN_MAPA_Y-16);
			cpct_drawSpriteMaskedAlignedTable (explosion.sprite, pvmem, 6, 16, g_tablatrans);
		}
		if(explosion2 == 1){
			pvmem = cpct_getScreenPtr(CPCT_VMEM_START, explosion.x+6, explosion.y+ORIGEN_MAPA_Y);
			cpct_drawSpriteMaskedAlignedTable (explosion.sprite, pvmem, 6, 16, g_tablatrans);
		}
		if(explosion3 == 1){
			pvmem = cpct_getScreenPtr(CPCT_VMEM_START, explosion.x, explosion.y+ORIGEN_MAPA_Y+16);
			cpct_drawSpriteMaskedAlignedTable (explosion.sprite, pvmem, 6, 16, g_tablatrans);
		} 
		if(explosion4 == 1){
			pvmem = cpct_getScreenPtr(CPCT_VMEM_START, explosion.x-6, explosion.y+ORIGEN_MAPA_Y);
			cpct_drawSpriteMaskedAlignedTable (explosion.sprite, pvmem, 6, 16, g_tablatrans);
		} 

		explosion.temporizador++;
	}
}
void borrarExplosion(){
	drawMapa();
}
/////////////////////////////////////DIBUJAR PERSONAJE\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\

void drawPj(){
	u8* memptr = cpct_getScreenPtr(CPCT_VMEM_START, personaje.x, personaje.y+ORIGEN_MAPA_Y);
	cpct_drawSpriteMaskedAlignedTable (personaje.frame->sprite, memptr, 8, 16, g_tablatrans);
}
void deletePj(){
	u8 w = 4 + (personaje.px & 1);
	u8 h = 4 + (personaje.py & 3 ? 1: 0);
	cpct_etm_drawTileBox2x4 (personaje.px / 2, personaje.py / 4, w,h, g_map1_W, ORIGEN_MAPA, mapa);
}
void reDrawPj(){
	deletePj();
	personaje.px = personaje.x;
	personaje.py = personaje.y;
	drawPj();
	}


////////////////////////////////////////////////////////////////////////////////////////////////AVANCE MAPA

void avanzarMapa(u8 nextmap, u8 pos) {
	u8 i;
	mapa_actual+=nextmap;
    mapa = array_mapas[mapa_actual];
	for ( i = 0; i < ENEMIGOS; i++){
		switch(mapa_actual){
			case 0:
				switch(i){
					case 0:
						enemigos[i]->x =1  ;
						enemigos[i]->y =20  ;
						enemigos[i]->muerto = NO;
					break;
					case 1:
						enemigos[i]->x = 20 ;
						enemigos[i]->y = 138 ;
						enemigos[i]->muerto = NO;
					break;
					case 2:
						enemigos[i]->muerto = SI;
					break;
					case 3:
						enemigos[i]->muerto = SI;
					break;
				}
			break;
			case 1:
				switch(i){
					case 0:
						enemigos[i]->x = 30 ;
						enemigos[i]->y = 30 ;
						enemigos[i]->muerto = NO;
					break;
					case 1:
						enemigos[i]->x = 30 ;
						enemigos[i]->y = 30 ;
						enemigos[i]->muerto = NO;
					break;
					case 2:
						enemigos[i]->muerto = SI;
					break;
					case 3:
						enemigos[i]->muerto = SI;
					break;
				}
			break;
			case 2:
				switch(i){
					case 0:
						enemigos[i]->muerto = SI;
					break;
					case 1:
						enemigos[i]->muerto = SI;
					break;
					case 2:
						enemigos[i]->x = 30  ;
						enemigos[i]->y = 30  ;
						enemigos[i]->muerto = NO;
					break;
					case 3:
						enemigos[i]->x = 30 ;
						enemigos[i]->y = 30  ;
						enemigos[i]->muerto = SI;
					break;
				}
			break;
		}
	}
	personaje.x = personaje.px = pos;
	personaje.move = 0;
	personaje.frame = &g_frames[8];
	personaje.nframe = 0;
	personaje.estado = personaje.lestado = ST_quieto;
   	drawMapa();
}
/////////////////////////////////////MOVERPERSONAJE\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\


void mover(u8 dir){
	u8  x1,x2,y1,y2,i;
	u8* tileSuelo1;
	u8* tileSuelo2;
	u8 col = 1;
	if     (dir == UP){	   x1 = UPLEFTX2; 	y1 = UPLEFTY2; 	 x2 = UPRIGHTX1; 	y2 = UPRIGHTY1;		}
	else if(dir == DOWN){  x1 = DOWNLEFTX1; y1 =DOWNLEFTY1; x2 = DOWNRIGHTX2;  y2 = DOWNRIGHTY2;   }
	else if(dir == LEFT){  x1 = UPLEFTX1; 	y1 = UPLEFTY1;	 x2 = DOWNLEFTX2; 	y2 = DOWNLEFTY2; 	}
	else if(dir == RIGHT){ x1 = UPRIGHTX2; 	y1 = UPRIGHTY2;	 x2 = DOWNRIGHTX1; 	y2 = DOWNRIGHTY1; 	}
	tileSuelo1 = obtenerTilePtr(personaje.x+x1, personaje.y+y1);
	tileSuelo2 = obtenerTilePtr(personaje.x+x2, personaje.y+y2);
	if(comprobarColision(tileSuelo1, tileSuelo2) == SI) {col = 0;}
	if(*tileSuelo1 == 54 || *tileSuelo1 == 55 || *tileSuelo1 == 56 || *tileSuelo1 == 63 || *tileSuelo1 == 64 || *tileSuelo1 == 65 || *tileSuelo1 == 72 || *tileSuelo1 == 73 || *tileSuelo1 == 74){
		if(llave.activo == NO)avanzarMapa(2,1);
	}
	if(dir == RIGHT && 
		        (( *tileSuelo1 == 57 || *tileSuelo1 == 79 || *tileSuelo1 == 61 
				|| *tileSuelo1 == 66 || *tileSuelo1 == 68 || *tileSuelo1 == 70 
				|| *tileSuelo1 == 75 || *tileSuelo1 == 77 || *tileSuelo1 == 79 )
			&& (   *tileSuelo2 == 57 || *tileSuelo2 == 59 || *tileSuelo2 == 61 
				|| *tileSuelo2 == 66 || *tileSuelo2 == 68 || *tileSuelo2 == 70 
				|| *tileSuelo2 == 75 || *tileSuelo2 == 77 || *tileSuelo2 == 79 ))){avanzarMapa(1,1);}
	else if( dir == LEFT &&
			    (( *tileSuelo1 == 58 || *tileSuelo1 == 60 || *tileSuelo1 == 62 
				|| *tileSuelo1 == 67 || *tileSuelo1 == 69 || *tileSuelo1 == 71 
				|| *tileSuelo1 == 76 || *tileSuelo1 == 78 || *tileSuelo1 == 80 )
			&& (   *tileSuelo2 == 58 || *tileSuelo2 == 60 || *tileSuelo2 == 62 
				|| *tileSuelo2 == 67 || *tileSuelo2 == 69 || *tileSuelo2 == 71 
				|| *tileSuelo2 == 76 || *tileSuelo2 == 78 || *tileSuelo2 == 80 ))){avanzarMapa(-1,70);}
	if(caja.activo == 1 && (personaje.x+x1 >= caja.x && personaje.x+x1 <= caja.x+5) && (personaje.y+y1+3 >= caja.y && personaje.y+y1+3 <= caja.y+16)){
			caja.activo = 0;
			personaje.bombas++;
			borrarCaja();
	}

	for (i = 0; i < 3; i++)
	{
		if(paredes[i]->activo == 1 && (personaje.x+x1 >= paredes[i]->x && personaje.x+x1 <= paredes[i]->x+5) && (personaje.y+y1+3 >= paredes[i]->y && personaje.y+y1+3 <= paredes[i]->y+16)){
				col = 0;
		}
	}

	if(mapa_actual == 0 && llave.activo == 1 && (personaje.x+x1 >= llave.x && personaje.x+x1 <= llave.x+5) && (personaje.y+y1+3+3 >= llave.y && personaje.y+y1+3 <= llave.y+16)){
			llave.activo = NO;
			borrarLlave();
	}
	if(col == 1){
   		if     (dir == LEFT) { personaje.x -= VELOCITYH;  personaje.estado = ST_andando_l;}
   		else if(dir == UP)   { personaje.y -= VELOCITYV;  personaje.estado = ST_andando_u;}
   		else if(dir == RIGHT){ personaje.x += VELOCITYH;  personaje.estado = ST_andando_r;}
   		else if(dir == DOWN) { personaje.y += VELOCITYV;  personaje.estado = ST_andando_d;}
   		if(personaje.estado != personaje.lestado){personaje.nframe = 0;personaje.lestado = personaje.estado;}
	}
}

void compruebaTeclado(){
	cpct_scanKeyboard_f();	
   if(cpct_isAnyKeyPressed()){
	   if ( cpct_isKeyPressed(Key_D))		mover(RIGHT);
	   else if ( cpct_isKeyPressed(Key_A)) mover(LEFT);
	   else if ( cpct_isKeyPressed(Key_W))	mover(UP);
	   else if ( cpct_isKeyPressed(Key_S)) mover(DOWN);	
	   if ( cpct_isKeyPressed(Key_Space) && bala.activo == NO )disparar();
	   if ( cpct_isKeyPressed(Key_B) && bomba.activo == NO && caja.activo == NO)ponerBomba();	
	}
}






/////////////////////////////////////MOVERENEMIGO\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\


u8 moverEn(u8 dir , Tenemigo* enemigo){
	u8  x1,x2,y1,y2,i;
	u8* tileSuelo1;
	u8* tileSuelo2;
	u8 col = 1;
	if     (dir == UP){	   x1 = UPLEFTX2; 	y1 = UPLEFTY2; 	 x2 = UPRIGHTX1; 	y2 = UPRIGHTY1;		}
	else if(dir == DOWN){  x1 = DOWNLEFTX1; y1 =DOWNLEFTY1;  x2 = DOWNRIGHTX2;  y2 = DOWNRIGHTY2;   }
	else if(dir == LEFT){  x1 = UPLEFTX1; 	y1 = UPLEFTY1;	 x2 = DOWNLEFTX2; 	y2 = DOWNLEFTY2; 	}
	else if(dir == RIGHT){ x1 = UPRIGHTX2; 	y1 = UPRIGHTY2;	 x2 = DOWNRIGHTX1; 	y2 = DOWNRIGHTY1; 	}
	tileSuelo1 = obtenerTilePtr(enemigo->x+x1, enemigo->y+y1);
	tileSuelo2 = obtenerTilePtr(enemigo->x+x2, enemigo->y+y2);
	if(enemigo->tipo == 1 && comprobarColision(tileSuelo1, tileSuelo2) == SI) {col = 0;}
	if(enemigo->tipo == 2 && ((enemigo->x+x1 < 2 && dir == LEFT)|| (enemigo->x+x1 > 70 && dir == RIGHT) || (enemigo->y+y1 < 260 && dir == UP ) || (enemigo->y+y1 > 155 && dir == DOWN) )){col = 0;}
	for (i = 0; i < 3; i++)
	{
		if(paredes[i]->activo == 1 && (enemigo->x+x1 >= paredes[i]->x && enemigo->x+x1 <= paredes[i]->x+5) && (enemigo->y+y1+3 >= paredes[i]->y && enemigo->y+y1+3 <= paredes[i]->y+16)){
				col = 0;
		}
	}
	if(col == 1){
   		enemigo->move = 1;
   		if 		(dir == LEFT) {  enemigo->x -= VELOCITYH; enemigo->estado = ST_enemy_andando_l;	}
   		else if (dir == UP)   {  enemigo->y -= VELOCITYV; enemigo->estado = ST_enemy_andando_u;	}
   		else if (dir == RIGHT){  enemigo->x += VELOCITYH; enemigo->estado = ST_enemy_andando_r;	}
   		else if (dir == DOWN) {  enemigo->y += VELOCITYV; enemigo->estado = ST_enemy_andando_d;	}
   		if(enemigo->estado != enemigo->lestado){enemigo->nframe = 0;enemigo->lestado = enemigo->estado;}
   		return SI;
	}
	else return NO;
}


/////////////////////////////////////////////IA DEL ENEMIGO///////////////////////////////////////////////
//EL ENEMIGO PATRUYA CON MOVIMIENTOS ESTABLECIDOS MEDIANTE CAMINOS RECTOS HASTA QUE CHOCA Y RECALCULA LA DIRECCION
//PARA DAR MAS SENSACION DE INTELIGENCIA GENERA UN NUMERO ALEATORIO CADA VEZ QUE SE DESPLAZA Y CON UNA PROBABILIDAD 
//DE 1/200 SE RECALCULA SU DIRECCION. 
//CUANDO EL PERSONAJE ENTRA EN SU AREA DE VISION EL ENEMIGO SE DETIENE Y SE DIRIJE EN LA DIRECCION EN LA QUE SE ENCUENTRA
//EL PERSONAJE, SI DURANTE SU TRAYECTO SE CHOCA, PARA DAR SENSACION DE INTELIGENCIA VUELVE DURANTE UNOS POCOS SEGUNDOS A
//SU ESTADO DE PATRUYA, DE ESTA MANERA SE DESPLAZARÁ A CUALQUIER DIRECCION Y CUANDO VUELVA A DARSE CUENTA DE QUE EL 
//PERSONAJE ESTA CERCA, HABRÁ CAMBIADO DE POSICION, HABIENDO POSIBILIDADES DE QUE DESDE ESTA NUEVA POSICION SEA CAPAZ DE 
//ALCANZAR AL PERSONAJE SIN CHOCAR.
//DE ESTA MANERA OBTENEMOS UNA IA SENCILLA Y QUE REQUIERE POCOS RECURSOS PERO DANDO LA SENSACION DE SER INTELIGENTE
////////////////////////////////////////////////////////////////////////////////////////////////////////////

u8 visionEnemigo(Tenemigo* enemigo){
	if(personaje.x - (enemigo->x - VISIONX) > 0){
		if(personaje.x - (enemigo->x + VISIONX) < 0 ){
			if(personaje.y - (enemigo->y - VISIONY) > 0){
				if(personaje.y - (enemigo->y + VISIONY) < 0){
					return SI;
				}
			}
		}
	}
	return NO;
}
void continuaMovimiento(Tenemigo* enemigo){
	if     (enemigo->estado == ST_enemy_andando_l){if(moverEn(LEFT, enemigo) == NO) { enemigo->pared = SI; }}
	else if(enemigo->estado == ST_enemy_andando_r){if(moverEn(RIGHT, enemigo) == NO){ enemigo->pared = SI; }}
	else if(enemigo->estado == ST_enemy_andando_u){if(moverEn(UP, enemigo) == NO)   { enemigo->pared = SI; }}
	else if(enemigo->estado == ST_enemy_andando_d){if(moverEn(DOWN, enemigo) == NO) { enemigo->pared = SI; }}
}
void patrulla(Tenemigo* enemigo){
	u8 rand;
	if(enemigo->siguiendo == SI) {enemigo->siguiendo = NO;}
	if(enemigo->pared == SI ){
		if     (enemigo->estado == ST_enemy_andando_l){if(moverEn(DOWN, enemigo) == NO) {if(moverEn(UP, enemigo) == NO)   {moverEn(RIGHT, enemigo);	}}}
		else if(enemigo->estado == ST_enemy_andando_r){if(moverEn(UP, enemigo) == NO) {if(moverEn(DOWN, enemigo) == NO)   {moverEn(LEFT, enemigo); 	}}}
		else if(enemigo->estado == ST_enemy_andando_u){if(moverEn(LEFT, enemigo) == NO) {if(moverEn(RIGHT, enemigo) == NO){moverEn(DOWN, enemigo); 	}}}
		else if(enemigo->estado == ST_enemy_andando_d){if(moverEn(RIGHT, enemigo) == NO){if(moverEn(LEFT, enemigo) == NO) {moverEn(UP, enemigo);   	}}}
		enemigo->pared = NO;
	}
	else{
		rand = cpct_rand8() % 100;
		if(rand == 0){enemigo->pared = SI;}
		else continuaMovimiento(enemigo);
	}
}
void persigue(Tenemigo* enemigo){
		enemigo->siguiendo = SI;
		if     (personaje.x - enemigo->x > 0){ if(moverEn(RIGHT, enemigo) == NO){ enemigo->contAlerta++; }}
		else if(personaje.x - enemigo->x < 0){ if(moverEn(LEFT, enemigo) == NO) { enemigo->contAlerta++; }}
		else if(personaje.y - enemigo->y > 0){ if(moverEn(DOWN, enemigo) == NO) { enemigo->contAlerta++; }}
		else if(personaje.y - enemigo->y < 0){ if(moverEn(UP, enemigo) == NO)   { enemigo->contAlerta++; }}
}
void mueveEnemigo(Tenemigo* enemigo){
	if(visionEnemigo(enemigo) == SI && enemigo->contAlerta == 0) persigue(enemigo);
	else {
		if(enemigo->contAlerta != NO && enemigo->contAlerta < enemigo->randAlerta){
			patrulla(enemigo);
			enemigo->contAlerta++;
		}
		else if(enemigo->contAlerta >= enemigo->randAlerta){
			enemigo->randAlerta = (cpct_rand8() % 200) + 50;
			enemigo->contAlerta = NO;
		}
		else patrulla(enemigo);
	}
}

	
//////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////
void resetPosiciones(){
	personaje.x = personaje.px = 1;
	personaje.y = personaje.py = 138;
	enemigo1.x = enemigo1.px = 1;
	enemigo1.y = enemigo1.py = 20;
	enemigo2.x = enemigo2.px = 60;
	enemigo2.y = enemigo2.py = 2;
}
u8 colisionConEnemigos(){
	u8 i;
	for (i = 0; i < ENEMIGOS; ++i){
		if(    personaje.x >= enemigos[i]->x-6  
			&& personaje.x <= enemigos[i]->x+6  
			&& personaje.y >= enemigos[i]->y-16
			&& personaje.y <= enemigos[i]->y+16){
			return SI;
		}
	}
	return NO;
}


/////////////////////////////////////LECTURA DE TECLADO Y DIBUJADO\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\

void game() {
	u8 i;
	cpct_clearScreen(0xFFFF);
	drawMapa();
	dibujarCorazones();
   while (1){
	  	compruebaTeclado();
	  	if(colisionConEnemigos() == NO){
  			for ( i = 0; i < ENEMIGOS; ++i){
  				if(enemigos[i]->muerto == NO){
  					mueveEnemigo(enemigos[i]);
  					stateEnExe(enemigos[i]);
  					selectEnSprite(enemigos[i]);
  				}
  			}
  			stateExe();
  			selectSprite();
  		   	cpct_waitVSYNC();
  		   	reDrawPj();
  		   	for ( i = 0; i < ENEMIGOS; ++i){
  				if(enemigos[i]->muerto == NO){
  		   			cpct_waitVSYNC();
  		   			reDrawEn(enemigos[i]);
  				}
  			}
  		   	if(mapa_actual == 0 && caja.activo == SI)dibujarCaja();
  		   	if( mapa_actual == 0) dibujarRompible();
  		   	if(llave.activo == SI && mapa_actual == 0)dibujarLlave();
  		   	if(bomba.activo == SI){dibujarBomba(); compruebaBomba();}
  		   	if(explosion.activo == SI){dibujarExplosion();}
  		   	if(bala.activo == SI){reDrawBala(); compruebaBala();}
	    }
	   	else{
			cpct_clearScreen(0xFFFF);
	   		personaje.vida -= 1;
	   		if(personaje.vida == 0){
    			mapa_actual = 0;
    			mapa = array_mapas[mapa_actual];
	   			personaje.vida = 3;
	   		}
	   		resetPosiciones();
	   		drawMapa();
			dibujarCorazones();
	   	}
	}
}

////////////////////////////INICIALIZAR CONFIGURACION, POSICIONES Y ESTADOS\\\\\\\\\\\\\\\\\\\\\\\\\\\\\

void init() {
	cpct_disableFirmware();
	cpct_setVideoMode(0);
	cpct_setBorder(HW_BLACK);
	cpct_setPalette(g_palette, 16);
	cpct_etm_setTileset2x4(g_tileset);
	mapa=g_map1;
	corazon.x = 0;
	corazon.y = 0;
	corazon.sprite = g_corazon;
	caja.x = 5;
	caja.y = 45;
	caja.sprite = g_cajab;
	caja.activo = 1;
	pared1.x = 47;
	pared1.y = 6;
	pared1.sprite = g_rompible;
	pared1.activo = SI;
	pared2.x = 65;
	pared2.y = 6;
	pared2.sprite = g_rompible;
	pared2.activo = SI;
	pared3.x = 47;
	pared3.y = 139;
	pared3.sprite = g_rompible;
	pared3.activo = SI;
	llave.x = 18;
	llave.y = 125;
	llave.sprite = g_llave;
	llave.activo = SI;
	bomba.activo = NO;
	bomba.sprite = g_bomba;
	explosion.activo = NO;
	explosion.sprite = g_explosion;
	bala.activo = NO;
	personaje.x = personaje.px =1;
	personaje.y = personaje.py = 138;
	personaje.move = NO;
	personaje.frame = &g_frames[8];
	personaje.nframe = 0;
	personaje.vida = 3;
	personaje.bombas = 29;
	personaje.muerto = NO;
	personaje.estado = personaje.lestado = ST_quieto;
	enemigo1.estado = enemigo1.lestado = ST_enemy_andando_r;
	enemigo2.estado = enemigo2.lestado = ST_enemy_andando_r;
	enemigo3.estado = enemigo3.lestado = ST_enemy_andando_r;
	enemigo4.estado = enemigo4.lestado = ST_enemy_andando_r;
	enemigo1.frame = &g_enemy_frames[8];
	enemigo2.frame = &g_enemy_frames[8];
	enemigo3.frame = &g_enemy2_frames[8];
	enemigo4.frame = &g_enemy2_frames[8];
	enemigo1.x = enemigo1.px =1;
	enemigo2.x = enemigo2.px = 60;
	enemigo1.y = enemigo1.py = 20;
	enemigo2.y = enemigo2.py = 138;
	enemigo1.contAlerta = NO;
	enemigo2.contAlerta = NO;
	enemigo3.contAlerta = NO;
	enemigo4.contAlerta = NO;
	enemigo1.siguiendo = NO;
	enemigo2.siguiendo = NO;
	enemigo3.siguiendo = NO;
	enemigo4.siguiendo = NO;
	enemigo1.muerto = NO;
	enemigo2.muerto = NO;
	enemigo3.muerto = SI;
	enemigo4.muerto = SI;
	enemigo1.nframe = 0;
	enemigo2.nframe = 0;
	enemigo3.nframe = 0;
	enemigo4.nframe = 0;
	enemigo1.pared = NO;
	enemigo2.pared = NO;
	enemigo3.pared = NO;
	enemigo4.pared = NO;
	enemigo1.move = NO;
	enemigo2.move = NO;
	enemigo3.move = NO;
	enemigo4.move = NO;
	enemigo1.tipo = 1;
	enemigo2.tipo = 1;
	enemigo3.tipo = 2;
	enemigo4.tipo = 2;
	enemigos[0] = &enemigo1;
	enemigos[1] = &enemigo2;
	enemigos[2] = &enemigo3;
	enemigos[3] = &enemigo4;
	paredes[0] = &pared1;
	paredes[1] = &pared2;
	paredes[2] = &pared3;
}

/////////////////////////////////////COMIENZA PROGRAMA\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\

void main(void) {
	init();
	game();
}
